#!/usr/bin/python

#############################################
#Author:        Damiano Barboni <barboni@meeo.it>
#Version:       
#Description:   Perform parallel read-only requests to RASDAMAN or PETASCOPE
#Usage:         ./test_parallel_request.py APP N
#                        APP can be "r" for RASDAMAN or "p" for PETASCOPE
#                        N is the number of requests generated
#Changelog:     None
#############################################

import httplib
import sys
import threading
import time
import commands

#global variables definition
WCS_host    = "earthserverclimate"
WCS_port    = 8080
WCS_url     = "/petascope/wcs2?"
WCS_service = "WCS"
WCS_version = "2.0.0"

CoverageId     = "NIR"
CollectionName = "NIR"

Output_File_Path = "/home/rasdaman/test"

class GetCoverage():
    #########################################
    ############## INIT CLASS ###############
    #########################################
    def __init__(self, **kw):
        #initlog
        self.Request = "GetCoverage"
        self.service = WCS_service
        self.url     = WCS_url
        self.subset  = ""
        
        #set coverage id
        self.CoverageId = kw['CoverageId']
          
        #set wcs version
        self.version = WCS_version
        
    #########################################
    ############# SET SUBSET METHOD #########
    #########################################
    def setSubset(self, axis, minVal, maxVal):
        #if the axis is different than x or y log a warning message becuse the axis was not tested 
        if axis.lower()!="x" and axis.lower()!="y" and axis.lower()!="t":
            self.LOG.warning("%s the axis %s is different than x-y-t and was not tested" %(self.Request, axis))
            #append new subset to self.subset string
        if axis.lower()=="t":
            self.subset += "subset%s=%s(%s,%s)&" %(axis.upper(), axis.lower(), minVal, maxVal)
        else:#axis.lower()=="x" OR axis.lower()=="y"
            self.subset += "subset%s=%s(%d,%d)&" %(axis.upper(), axis.lower(), minVal, maxVal)
        
        return self.subset
    
    #########################################
    ############ Get data METHOD ############
    #########################################
    
    def getData(self, **kw):
        #get host coverage id
        host = kw['host']
        port = kw['port']
        
        #create string
        request_string = WCS_url + "service=WCS" + "&Request=" + self.Request + "&version=" + self.version + "&CoverageId=" + self.CoverageId + "&" + self.subset
        
        conn = WCSRequestGet(host, port)
        xml = conn.execute(request_string)
        
        return xml
         
#implement http reguest to WCS server
class WCSRequestGet():
    def __init__(self, host, port = WCS_port):
        self.host = host
        self.port = port
        
    def execute(self, wcs_request_string):
        #init connection
        conn = httplib.HTTPConnection(self.host+":"+str(self.port))
        conn.request("GET", wcs_request_string)
        r = conn.getresponse()
        
        #check if response status is 200
        r_status = r.status
        if 200 <= r_status <= 299:
            xml = r.read()
        else:
            xml = "ERROR " %(r_status)
            print ("Unable to get data from WCS server. Bad response status N %d returned by WCS server" %(r_status))
        
        conn.close()                                
        return xml
        
class requestThread(threading.Thread):
    def __init__(self,application, i):
        threading.Thread.__init__(self)
        self.application = application
        self.th_number   = i
    
    #run method       
    def run(self):
        start = time.time()
        print "Thread %d STARTED" %(self.th_number)
        if self.application == "r":
            #request to rasdaman
            rasdaman_request = 'rasql -q "select csv(n[0:200,0:200]) from NIR as n" --out string'
            data_Out      =  commands.getstatusoutput(rasdaman_request)
            
            filename = "%s/rasdaman_output_%d.txt" %(Output_File_Path,self.th_number)
            
        elif application == "p":
            #request to petascope
            obj = GetCoverage(CoverageId=CoverageId)
            #set subset
            obj.setSubset("x", 0,200)
            obj.setSubset("y", 0,200)
    
            #get xml response
            data_Out = obj.getData(host=WCS_host, port=WCS_port)
            
            #save file
            filename = "%s/petascope_output_%d.xml" %(Output_File_Path,self.th_number)
            
        #write output to file
        out_file = open(filename,"w")
        out_file.write(data_Out)
        out_file.close()
            
        print "Thread %d ENDED in %f seconds" %(self.th_number,time.time()-start)
        sys.exit(0)
    
if __name__ == '__main__':
    try:
        application = sys.argv[1]
        req_number = int(sys.argv[2],10)
        if (application != "r" or application != "p") and req_number < 1:
            raise
    except:
        print "ERROR bad arguments. USAGE:\n\tpython %s APP N" %(sys.argv[0])
        print '\t\tAPP can be "r" for RASDAMAN or "p" for PETASCOPE'
        print '\t\tN is the number of requests generated'
        sys.exit(65)
    
    #thread generation
    thread_list = []
    for i in range(0, req_number):
        thread_list.append(requestThread(application, i))
        #thread execution
        thread_list[i].start()
        
    #Wait for the thread_list tasks to finish
    for i in range(0,len(thread_list)):
        thread_list[i].join()
        
    print "COMPLETE!"
    
    sys.exit(0)
    
